/**
  * This sketch demonstrates how to use an FFT to analyze
  * the audio being generated by an AudioPlayer.
  * <p>
  * FFT stands for Fast Fourier Transform, which is a 
  * method of analyzing audio that allows you to visualize 
  * the frequency content of a signal. You've seen 
  * visualizations like this before in music players 
  * and car stereos.
  * <p>
  * For more information about Minim and additional features, 
  * visit http://code.compartmental.net/minim/
  */

import ddf.minim.analysis.*;
import ddf.minim.*;

Minim       minim;
AudioPlayer sample;
FFT         fft;

StringList dataTable;

void setup(){
  size(512, 200, P3D);
  
  minim = new Minim(this);
  
  // specify that we want the audio buffers of the AudioPlayer
  // to be 1024 samples long because our FFT needs to have 
  // a power-of-two buffer size and this is a good size.
  sample = minim.loadFile("walk-0.mp3", 1024);
  
  // loop the file indefinitely
  sample.loop();
  
  // create an FFT object that has a time-domain buffer 
  // the same size as sample's sample buffer
  // note that this needs to be a power of two 
  // and that it means the size of the spectrum will be half as large.
  fft = new FFT( sample.bufferSize(), sample.sampleRate() );
  
  dataTable = new StringList();
  
}

void draw(){
  background(0);
  stroke(255);
  
  // perform a forward FFT on the samples in sample's mix buffer,
  // which contains the mix of both the left and right channels of the file
  fft.forward( sample.mix );
  
  String dataRow = "";
  
  for(int i = 0; i < fft.specSize(); i++){    
    dataRow = dataRow + "," + fft.getBand(i);
    // draw the line for frequency band i, scaling it up a bit so we can see it
    line( i, height,  i, height - fft.getBand(i)*60);
  }
  
  dataTable.append(dataRow);
  
  println(sample.position() + " - " + sample.length());
  
  if (sample.position()== sample.length()){
    String tData[] = new String [dataTable.size()];
    for (int j=0; j<dataTable.size(); j++){
      tData[j] = dataTable.get(j);
    }

    saveStrings("data.csv", tData);
    
    exit();
  }
 }
